import errno

import cephfs

from ..exception import VolumeException


def _strtobool(val):
    """Convert a string representation of truth to true (1) or false (0).

    True values are 'y', 'yes', 't', 'true', 'on', and '1'; false values
    are 'n', 'no', 'f', 'false', 'off', and '0'.  Raises ValueError if
    'val' is anything else.
    """
    # Copied from distutils
    val = val.lower()
    if val in ('y', 'yes', 't', 'true', 'on', '1'):
        return 1
    elif val in ('n', 'no', 'f', 'false', 'off', '0'):
        return 0
    else:
        raise ValueError("invalid truth value %r" % (val,))


_pin_value = {
    "export": lambda x: int(x),
    "distributed": lambda x: int(_strtobool(x)),
    "random": lambda x: float(x),
}
_pin_xattr = {
    "export": "ceph.dir.pin",
    "distributed": "ceph.dir.pin.distributed",
    "random": "ceph.dir.pin.random",
}

def pin(fs, path, pin_type, pin_setting):
    """
    Set a pin on a directory.
    """
    assert pin_type in _pin_xattr

    try:
        pin_setting = _pin_value[pin_type](pin_setting)
    except ValueError:
        raise VolumeException(-errno.EINVAL, f"pin value wrong type: {pin_setting}")

    try:
        fs.setxattr(path, _pin_xattr[pin_type], str(pin_setting).encode('utf-8'), 0)
    except cephfs.Error as e:
        raise VolumeException(-e.args[0], e.args[1])
